/**
 * @author yutent<yutent.io@gmail.com>
 * @date 2020/09/16 11:54:31
 */

/**
 * 重写原生的flat方法, 性能提升5~10倍(看数组的结构)
 */
Object.defineProperty(Array.prototype, 'flat', {
  value: function (deep = 1, arr = []) {
    for (let it of this) {
      if (Array.isArray(it) && deep > 0) {
        it.flat(deep - 1, arr)
      } else {
        arr.push(it)
      }
    }

    return arr
  },
  writable: true
})

Object.defineProperty(Array.prototype, 'flatMap', {
  value: function (fn) {
    return this.map(fn).flat()
  },
  writable: true
})

if (!Array.prototype.at) {
  Object.defineProperty(Array.prototype, 'at', {
    value: function (num) {
      let n = +num
      if (n < 0) {
        n += this.length
      }
      return this[n]
    },
    writable: true
  })
}

if (!Array.prototype.findLast) {
  Object.defineProperty(Array.prototype, 'findLast', {
    value: function (fn) {
      let num = this.length
      while (num > 0) {
        let item = this[--num]
        if (fn(item, num)) {
          return item
        }
      }
    },
    writable: true
  })

  // 没有findLast也不会有findLastIndex
  Object.defineProperty(Array.prototype, 'findLastIndex', {
    value: function (fn) {
      let num = this.length
      while (num > 0) {
        let item = this[--num]
        if (fn(item, num)) {
          return num
        }
      }
      return -1
    },
    writable: true
  })
}

// 最新增加的几种方法
if (!Array.prototype.toSorted) {
  Object.defineProperty(Array.prototype, 'toSorted', {
    value: function (fn) {
      return [...this].sort(fn)
    },
    writable: true
  })

  Object.defineProperty(Array.prototype, 'toReversed', {
    value: function () {
      return [...this].reverse()
    },
    writable: true
  })

  Object.defineProperty(Array.prototype, 'toSpliced', {
    value: function (...args) {
      return [...this].splice(...args)
    },
    writable: true
  })

  Object.defineProperty(Array.prototype, 'with', {
    value: function (num, val) {
      let n = +num
      let arr = [...this]
      if (n < 0) {
        n += arr.length
      }
      arr[n] = val
      return arr
    },
    writable: true
  })
}

// 数组分组
if (!Array.prototype.group) {
  Object.defineProperty(Array.prototype, 'group', {
    value: function (fn) {
      let output = {}
      if (typeof fn === 'function') {
        for (let it of this) {
          let key = fn(it)
          if (output[key]) {
            output[key].push(it)
          } else {
            output[key] = [it]
          }
        }
        return output
      } else {
        throw Error('argument callback must be a function.')
      }
    },
    writable: true
  })

  Object.defineProperty(Array.prototype, 'groupToMap', {
    value: function (fn) {
      let output = new Map()
      if (typeof fn === 'function') {
        for (let it of this) {
          let key = fn(it)
          if (output.has(key)) {
            output.get(key).push(it)
          } else {
            output.set(key, [it])
          }
        }
        return output
      } else {
        throw Error('argument callback must be a function.')
      }
    },
    writable: true
  })
}
