/**
 * @author yutent<yutent.io@gmail.com>
 * @date 2020/09/16 12:09:15
 */

// 版本号比较, 返回 -1, 0, 1
function compare(v1, v2) {
  v1 += ''
  v2 += ''

  if (v1 === v2) {
    return 0
  } else {
    v1 = v1.split('.')
    v2 = v2.split('.')

    let max = Math.max(v1.length, v2.length)

    for (let i = 0; i < max; i++) {
      let _1 = +v1[i] || 0,
        _2 = +v2[i] || 0

      if (_1 > _2) {
        return 1
      } else if (_1 < _2) {
        return -1
      }
    }

    return 0
  }
}

//类似于Array 的splice方法
if (!String.prototype.splice) {
  Object.defineProperty(String.prototype, 'splice', {
    value: function (start, len, fill) {
      let length = this.length
      let argLen = arguments.length

      fill = fill === undefined ? '' : fill

      if (argLen < 1) {
        return this
      }

      //处理负数
      if (start < 0) {
        if (Math.abs(start) >= length) {
          start = 0
        } else {
          start = length + start
        }
      }

      if (argLen === 1) {
        return this.slice(0, start)
      } else {
        len -= 0

        let strl = this.slice(0, start)
        let strr = this.slice(start + len)

        return strl + fill + strr
      }
    }
  })
}

//同php的htmlspecialchars函数
if (!String.prototype.htmlspecialchars) {
  Object.defineProperty(String.prototype, 'htmlspecialchars', {
    value: function (sign) {
      let str = this.replace(/&(?!\w+;)/g, '&amp;')
        .replace(/</g, '&lt;')
        .replace(/>/g, '&gt;')

      if (sign === 'ENT_QUOTES') {
        return str.replace(/"/g, '&quot;').replace(/'/g, '&#39;')
      } else if (sign === 'ENT_NOQUOTES') {
        return str
      } else {
        return str.replace(/"/g, '&quot;')
      }
    }
  })
}

//htmlspecialchars的还原
if (!String.prototype.tohtml) {
  Object.defineProperty(String.prototype, 'tohtml', {
    value: function () {
      return this.replace(/&lt;/gi, '<')
        .replace(/&gt;/gi, '>')
        .replace(/&quot;/gi, '"')
        .replace(/&#39;/g, "'")
        .replace(/&amp;/gi, '&')
    }
  })
}

//简单的过滤xss
if (!String.prototype.xss) {
  Object.defineProperty(String.prototype, 'xss', {
    value: function () {
      let str = this.htmlspecialchars('ENT_QUOTES')
      str = str
        .replace(
          /(document\.cookie)|(document\.write)|(\.parentNode)|(window\.location)|(\.innerHTML)/g,
          ''
        )
        .replace(/(%0[0-8bcef])|(%1[0-9a-f])/g, '')
      return str
    }
  })
}

// js特殊字符的转义
if (!String.prototype.escape) {
  Object.defineProperty(String.prototype, 'escape', {
    value: function () {
      return this.replace(/('|"|&|\\|\}|\{|\(|\)|;|=|\,|&)/g, '\\$1')
    }
  })
}

if (!String.prototype.at) {
  Object.defineProperty(String.prototype, 'at', {
    value: function (num) {
      let n = +num
      if (n < 0) {
        n += this.length
      }
      return this[n]
    },
    writable: true
  })
}

/**
 * 将url参数转为对象
 * 注意: 这里不处理同一个参数多次出现的情况, 会直接覆盖
 */
if (!String.prototype.toJson) {
  Object.defineProperty(String.prototype, 'toJson', {
    value: function () {
      let str = this.replace(/^\?/, '')
      let params = decodeURIComponent(str)
        .split('&')
        .filter(_ => _)
        .map(_ => _.split('='))
      let obj = {}
      for (let it of params) {
        obj[it[0]] = it[1] || ''
      }
      return obj
    }
  })
}

// 用于版本号的比较
if (!String.prototype.lt) {
  Object.defineProperty(String.prototype, 'lt', {
    value: function (v) {
      return compare(this, v) === -1
    }
  })

  Object.defineProperty(String.prototype, 'lte', {
    value: function (v) {
      return compare(this, v) < 1
    }
  })

  Object.defineProperty(String.prototype, 'gt', {
    value: function (v) {
      return compare(this, v) === 1
    }
  })

  Object.defineProperty(String.prototype, 'gte', {
    value: function (v) {
      return compare(this, v) > -1
    }
  })

  Object.defineProperty(String.prototype, 'eq', {
    value: function (v) {
      return compare(this, v) === 0
    }
  })
}
